@@ -33,6 +33,7 @@
 #include <libgnome/libgnome.h>
 #include <libgnomeui/gnome-ui-init.h>
 /* ---------------------------------- */
+#include <libgnomeui/gnome-thumbnail.h>
 
 static AppearanceData *
 init_appearance_data (int *argc, char ***argv)
@@ -60,6 +61,7 @@ init_appearance_data (int *argc, char ***argv)
     data = g_new (AppearanceData, 1);
     data->client = gconf_client_get_default ();
     data->xml = ui;
+    data->thumb_factory = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
   }
 
   return data;
@@ -76,9 +78,11 @@ main_window_response (GtkWidget *widget,
     gtk_main_quit ();
 
     themes_shutdown (data);
+    style_shutdown (data);
     desktop_shutdown (data);
     font_shutdown (data);
 
+    g_object_unref (data->thumb_factory);
     g_object_unref (data->client);
     g_object_unref (data->xml);
   }
@@ -93,6 +97,7 @@ main (int argc, char **argv)
 
   gchar *install_filename = NULL;
   gchar *start_page = NULL;
+  gchar **wallpaper_files = NULL;
   GOptionContext *option_context;
   GOptionEntry option_entries[] = {
       { "install-theme",
@@ -107,8 +112,16 @@ main (int argc, char **argv)
         G_OPTION_FLAG_IN_MAIN,
         G_OPTION_ARG_STRING,
         &start_page,
+        /* TRANSLATORS: don't translate the terms in brackets */
         N_("Specify the name of the page to show (theme|background|fonts|interface)"),
         N_("page") },
+      { G_OPTION_REMAINING,
+      	0,
+      	G_OPTION_FLAG_IN_MAIN,
+      	G_OPTION_ARG_FILENAME_ARRAY,
+      	&wallpaper_files,
+      	NULL,
+      	N_("[WALLPAPER...]") },
       { NULL }
     };
 
@@ -137,7 +150,8 @@ main (int argc, char **argv)
   /* init tabs */
   themes_init (data);
   style_init (data);
-  desktop_init (data);
+  desktop_init (data, (const gchar **) wallpaper_files);
+  g_strfreev (wallpaper_files);
   font_init (data);
   ui_init (data);
 
@@ -146,8 +160,12 @@ main (int argc, char **argv)
   capplet_set_icon (w, "gnome-settings-theme");
   gtk_widget_show_all (w);
 
-  g_signal_connect_after (G_OBJECT (w), "response",
-                          G_CALLBACK (main_window_response), data);
+  g_signal_connect_after (w, "response",
+                          (GCallback) main_window_response, data);
+
+  /* default to background page if files were given on the command line */
+  if (wallpaper_files && !install_filename && !start_page)
+    start_page = g_strdup ("background");
 
   if (start_page != NULL) {
     gchar *page_name;
