@@ -136,8 +136,10 @@ theme_get_thumbnail_from_cache (GnomeThemeMetaInfo *info, AppearanceData *data)
                                                    path, mtime);
   g_free (path);
 
-  if (thumb_filename != NULL)
+  if (thumb_filename != NULL) {
     thumb = gdk_pixbuf_new_from_file (thumb_filename, NULL);
+    g_free (thumb_filename);
+  }
 
   return thumb;
 }
@@ -223,6 +225,8 @@ theme_load_from_gconf (GConfClient *client, GnomeThemeMetaInfo *theme)
 
   g_free (theme->gtk_theme_name);
   theme->gtk_theme_name = gconf_client_get_string (client, GTK_THEME_KEY, NULL);
+  if (theme->gtk_theme_name == NULL)
+    theme->gtk_theme_name = g_strdup ("Clearlooks");
 
   g_free (theme->gtk_color_scheme);
   scheme = gconf_client_get_string (client, COLOR_SCHEME_KEY, NULL);
@@ -235,18 +239,26 @@ theme_load_from_gconf (GConfClient *client, GnomeThemeMetaInfo *theme)
 
   g_free (theme->metacity_theme_name);
   theme->metacity_theme_name = gconf_client_get_string (client, METACITY_THEME_KEY, NULL);
+  if (theme->metacity_theme_name == NULL)
+    theme->metacity_theme_name = g_strdup ("Clearlooks");
 
   g_free (theme->icon_theme_name);
   theme->icon_theme_name = gconf_client_get_string (client, ICON_THEME_KEY, NULL);
+  if (theme->icon_theme_name == NULL)
+    theme->icon_theme_name = g_strdup ("gnome");
 
   g_free (theme->cursor_theme_name);
-
 #ifdef HAVE_XCURSOR
   theme->cursor_theme_name = gconf_client_get_string (client, CURSOR_THEME_KEY, NULL);
   theme->cursor_size = gconf_client_get_int (client, CURSOR_SIZE_KEY, NULL);
 #else
   theme->cursor_theme_name = gconf_client_get_string (client, CURSOR_FONT_KEY, NULL);
 #endif
+  if (theme->cursor_theme_name == NULL)
+    theme->cursor_theme_name = g_strdup ("default");
+
+  g_free (theme->application_font);
+  theme->application_font = gconf_client_get_string (client, APPLICATION_FONT_KEY, NULL);
 }
 
 static gchar *
@@ -358,30 +370,28 @@ theme_set_custom_from_theme (const GnomeThemeMetaInfo *info, AppearanceData *dat
     g_free (custom->icon_theme_name);
     g_free (custom->metacity_theme_name);
     g_free (custom->gtk_color_scheme);
-    custom->gtk_theme_name = NULL;
-    custom->icon_theme_name = NULL;
-    custom->metacity_theme_name = NULL;
+    g_free (custom->cursor_theme_name);
+    g_free (custom->application_font);
     custom->gtk_color_scheme = NULL;
+    custom->application_font = NULL;
 
-    if (info->gtk_theme_name)
-      custom->gtk_theme_name = g_strdup (info->gtk_theme_name);
-    else
-      custom->gtk_theme_name = get_default_string_from_key (data->client, GTK_THEME_KEY);
-
-    if (info->icon_theme_name)
-      custom->icon_theme_name = g_strdup (info->icon_theme_name);
-    else
-      custom->icon_theme_name = get_default_string_from_key (data->client, ICON_THEME_KEY);
-
-    if (info->metacity_theme_name)
-      custom->metacity_theme_name = g_strdup (info->metacity_theme_name);
-    else
-      custom->metacity_theme_name = get_default_string_from_key (data->client, METACITY_THEME_KEY);
+    /* these settings are guaranteed to be non-NULL */
+    custom->gtk_theme_name = g_strdup (info->gtk_theme_name);
+    custom->icon_theme_name = g_strdup (info->icon_theme_name);
+    custom->metacity_theme_name = g_strdup (info->metacity_theme_name);
+    custom->cursor_theme_name = g_strdup (info->cursor_theme_name);
+    custom->cursor_size = info->cursor_size;
 
+    /* these can be NULL */
     if (info->gtk_color_scheme)
       custom->gtk_color_scheme = g_strdup (info->gtk_color_scheme);
     else
       custom->gtk_color_scheme = get_default_string_from_key (data->client, COLOR_SCHEME_KEY);
+
+    if (info->application_font)
+      custom->application_font = g_strdup (info->application_font);
+    else
+      custom->application_font = get_default_string_from_key (data->client, APPLICATION_FONT_KEY);
   }
 
   /* select the custom theme */
@@ -734,10 +744,17 @@ theme_store_sort_func (GtkTreeModel *model,
 
     gtk_tree_model_get (model, b, COL_NAME, &b_name, COL_LABEL, &b_label, -1);
 
-    if (!strcmp (b_name, CUSTOM_THEME_NAME))
+    if (!strcmp (b_name, CUSTOM_THEME_NAME)) {
       rc = 1;
-    else
-      rc = strcmp (a_label, b_label);
+    } else {
+      gchar *a_case, *b_case;
+
+      a_case = g_utf8_casefold (a_label, -1);
+      b_case = g_utf8_casefold (b_label, -1);
+      rc = g_utf8_collate (a_case, b_case);
+      g_free (a_case);
+      g_free (b_case);
+    }
 
     g_free (b_name);
     g_free (b_label);
@@ -864,7 +881,9 @@ themes_init (AppearanceData *data)
   renderer = gtk_cell_renderer_text_new ();
   g_object_set (renderer, "alignment", PANGO_ALIGN_CENTER,
 			  "wrap-mode", PANGO_WRAP_WORD_CHAR,
-			  "xalign", 0.5, "yalign", 0.0, NULL);
+			  "wrap-width", gtk_icon_view_get_item_width (icon_view),
+			  "width", gtk_icon_view_get_item_width (icon_view),
+			  "xalign", 0.0, "yalign", 0.0, NULL);
   gtk_cell_layout_pack_end (GTK_CELL_LAYOUT (icon_view), renderer, FALSE);
   gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (icon_view), renderer,
                                   "markup", COL_LABEL, NULL);
